﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace GanttChartDataGridMvvmSample.Models
{
    // Represents the task model object.
    public class Task : INotifyPropertyChanged
    {
        private string name;
        public string Name { get { return name; } set { name = value; OnPropertyChanged("Name"); } }

        private DateTime start;
        public DateTime Start { get { return start; } set { start = value; OnPropertyChanged("Start"); } }

        private DateTime finish;
        public DateTime Finish { get { return finish; } set { finish = value; OnPropertyChanged("Finish"); OnPropertyChanged("IsCompleted"); } }

        private DateTime completedFinish;
        public DateTime CompletedFinish { get { return completedFinish; } set { completedFinish = value; OnPropertyChanged("CompletedFinish"); OnPropertyChanged("IsCompleted"); } }

        public bool IsCompleted { get { return CompletedFinish >= Finish; } set { CompletedFinish = Finish; } }

        // Optionally, provide a better implementation for partial completion, considering only working time.
        public double Completion { get { return Math.Max(0, (CompletedFinish - Start).TotalHours / (Finish - Start).TotalHours); } set { CompletedFinish = Start + TimeSpan.FromHours(value * (Finish - Start).TotalHours); } }

        private string assignments;
        public string Assignments { get { return assignments; } set { assignments = value; OnPropertyChanged("Assignments"); } }
        
        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}
