﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using GanttChartDataGridMvvmSample.Models;
using System.Windows.Media;

namespace GanttChartDataGridMvvmSample.ViewModels
{
    // Represents the main view-model object of the application.
    public class ProjectViewModel : INotifyPropertyChanged
    {
        // References the model object to be displayed in the bound project views.
        public Project Model { get; set; }

        #region Task selection

        // Provides task selection by index, as views require.
        public int SelectedTaskIndex
        {
            get { return Model != null && Model.Tasks != null ? Model.Tasks.IndexOf(SelectedTask) : -1; }
            set
            {
                if (Model == null || Model.Tasks == null)
                    return;
                SelectedTask = value >= 0 && value < Model.Tasks.Count ? Model.Tasks[value] : null;
            }
        }

        // Provides task selection by reference.
        private Task selectedTask;
        public Task SelectedTask
        {
            get { return selectedTask; }
            set
            {
                if (Model == null || Model.Tasks == null)
                    return;
                selectedTask = value; OnPropertyChanged("SelectedTask"); OnPropertyChanged("SelectedTaskIndex"); OnPropertyChanged("SelectedTaskViewModel");
            }
        }

        // Provides a task view-model object for the selected task.
        // Optionally, you may improve this implementation by using a cache technique.
        public TaskViewModel SelectedTaskViewModel { get { return SelectedTask != null ? new TaskViewModel { Model = SelectedTask } : null; } }

        #endregion

        #region Settings

        // Provides a user interface setting that shows the entire timeline page zoomed out in the chart view.
        private bool fitToTimeline;
        public bool FitToTimeline { get { return fitToTimeline; } set { fitToTimeline = value; OnPropertyChanged("FitToTimeline"); } }

        // Provides a user interface setting that allows changing the color of task bars in the chart view.
        private string barColorName = "Blue";
        public string BarColorName { get { return barColorName; } set { barColorName = value; OnPropertyChanged("BarColorName"); OnPropertyChanged("BarBrush"); } }
        public Brush BarBrush { get { return new SolidColorBrush((Color)ColorConverter.ConvertFromString(BarColorName)); } }

        #endregion

        #region Operations

        // Adds a new task to the project model.
        public Task AddNewTask()
        {
            if (Model == null || Model.Tasks == null)
                return null;
            var task = new Task { Name = "New task", Start = DateTime.Today.Add(TimeSpan.Parse("08:00")), Finish = DateTime.Today.Add(TimeSpan.Parse("16:00")) };
            task.Completion = 0;
            Model.Tasks.Add(task);
            SelectedTask = task;
            return task;
        }

        // Deletes the selected task from the project model.
        public bool DeleteSelectedTask()
        {
            if (Model == null || Model.Tasks == null || SelectedTask == null)
                return false;
            return Model.Tasks.Remove(SelectedTask);
        }
        public bool CanDeleteSelectedTask()
        {
            return SelectedTask != null;
        }

        #endregion

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}
