﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using GanttChartDataGridMvvmSample.Models;
using System.Windows.Media;

namespace GanttChartDataGridMvvmSample.ViewModels
{
    // Represents the task view-model object of the application.
    public class TaskViewModel : INotifyPropertyChanged
    {
        // References the model object to be displayed in the bound task view.
        private Task model;
        public Task Model
        {
            get { return model; }
            set
            {
                if (model != null)
                    model.PropertyChanged -= Model_PropertyChanged;
                model = value;
                if (model != null)
                    model.PropertyChanged += Model_PropertyChanged;
            }
        }
        private void Model_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "Completion":
                    OnPropertyChanged("PercentCompleted");
                    break;
            }
        }

        #region Computed values

        // Provides percent completed value editable from the user interface.
        public double PercentCompleted
        {
            get { return Model.Completion * 100; }
            set
            {
                if (Model == null)
                    return;
                Model.Completion = value / 100;
            }
        }

        #endregion

        #region Operations

        // Sets the current task as completed.
        // Completed amount parameter should be between 0 and 1.
        public void SetAsCompleted(double completedAmount = 1)
        {
            if (Model == null)
                return;
            Model.Completion = completedAmount;
        }

        #endregion

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}
